/* S-expression scanner. */
%{
 #include "common.h"
%}

space       [ \t\n]+

alpha       [A-Za-z]
digit       [0-9] 

identifier  {alpha}({alpha}|{digit}|_)*
integer     {digit}+
floating    {digit}+[\.]{digit}+
comment     \/\/[^\n]*\n

%option noyywrap

%%

{integer}     { return INTVAL; }
{floating}    { return DBLVAL; }
[\[]          { return LBRACKET; }
[\]]          { return RBRACKET; }
[(]           { return LPAREN; }
[)]           { return RPAREN; }
[:]           { return COLON; }
[@]           { return AT; }
$d{integer}   { return DBLARG; }
$b{integer}   { return BOOLARG; }
{space}       ;
{comment}     ;
true          { return BOOLVAL; }
false         { return BOOLVAL; }
{identifier}  { return FUNCTION; }
<<EOF>>       { return FILEEND; }
.             { return ERROR; }

%%

