/************************************************************************
 *
 *   File: benchmark_info.cc:
 *   
 *   Author: Mark Stephenson
 *   Date: Mon Sep  2 20:10:20 2002
 *
 *   Function: A class that encapsulates benchmarks and their input
 *       files.
 *
 ***********************************************************************/

#include "benchmark_info.h"
#include <math.h>
#include <bits/nan.h>
#include <stdio.h>


int benchmark_info::k_bid = 0;


benchmark_info::benchmark_info () :
  m_setup_commands (NULL),
  m_profile_commands (NULL),
  m_compile_commands (NULL),
  m_link_commands (NULL),
  m_run_commands (NULL),
  m_check_commands (NULL),
  m_clean_commands (NULL),
  m_name (""),
  m_timeout (60000),
  m_optimizability (0.0),
  m_recency (1.0)
{
  m_bid = k_bid++;
}  


benchmark_info::~benchmark_info ()
{
  if (m_setup_commands) delete[] m_setup_commands;
  if (m_profile_commands) delete[] m_profile_commands;
  if (m_compile_commands) delete[] m_compile_commands;
  if (m_link_commands) delete[] m_link_commands;
  if (m_run_commands) delete[] m_run_commands;
  if (m_check_commands) delete[] m_check_commands;
  if (m_clean_commands) delete[] m_clean_commands;
}


/*************************************************************
 *
 *  Returns the length of the variable.
 *
 ************************************************************/
unsigned
benchmark_info::variable_length (char *str) const
{
  unsigned pos = 1;

  while (pos++) 
  {
    char ch = str[pos];
    if (ch == '/' || ch == ' ' || ch == '\0' || 
	ch == '\n' || ch == ';' || ch == '.')
      return pos;
  }

  return pos;
}


/*************************************************************
 *
 *  Expands the variables in the command lists.
 *
 ************************************************************/
void
benchmark_info::expand_variable_definitions ()
{
  expand_commands (m_setup_commands);
  expand_commands (m_profile_commands);
  expand_commands (m_compile_commands);
  expand_commands (m_link_commands);
  expand_commands (m_run_commands);
  expand_commands (m_check_commands);
  expand_commands (m_clean_commands);
}


/*************************************************************
 *
 *  Expands the variables in the string that is passed in.
 *
 ************************************************************/
void
benchmark_info::expand_string (string& str)
{
  int pos = 0;
  unsigned len;
  string var;
  char *cptr;

  pos = str.find ("$", pos);
  while (pos != string::npos)
  {
    cptr = (char*) str.c_str();
    len = variable_length (cptr + pos);
    var = str.substr (pos, len);
    str.replace (pos, len, m_env[var]);
    pos = str.find ("$", pos);
  }
}


/*************************************************************
 *
 *  Executes all the commands in the command list.  Variable
 *  names must be expanded before the commands are executed.
 *
 ************************************************************/
void
benchmark_info::expand_commands (command_list *cmd_list)
{
  command_list *beg = cmd_list;
  while (beg != NULL) 
  {
    expand_string (beg->entry);
    beg = beg->next;
  }
}
