/************************************************************************
 *
 *   File: benchmark_info.h:
 *   
 *   Author: Mark Stephenson
 *   Date: Tue Aug 20 14:07:09 2002
 *
 *   Function: A class that encapsulates benchmarks and their input
 *       files.
 *
 ***********************************************************************/

#ifndef __BENCHMARK_INFO_H__
#define __BENCHMARK_INFO_H__

#include <vector>
#include <string>
#include <map>

struct command_list
{
  string entry;
  command_list *next;
};

typedef map<string,string> environment;

class benchmark_info
{
 public:

  benchmark_info ();
  virtual ~benchmark_info ();

  void add_to_environment (const string& varname, const string& value) 
    { m_env[varname] = value; }

  // Sets the various command lists.
  void set_setup_commands   (command_list *list) { m_setup_commands = list; }
  void set_profile_commands (command_list *list) { m_profile_commands = list; }
  void set_compile_commands (command_list *list) { m_compile_commands = list; }
  void set_link_commands    (command_list *list) { m_link_commands = list; }
  void set_run_commands     (command_list *list) { m_run_commands = list; }
  void set_check_commands   (command_list *list) { m_check_commands = list; }
  void set_clean_commands   (command_list *list) { m_clean_commands = list; }

  // Get the various command lists.  For the lists to be directly executable,
  // expand_variable_definitions() should be called first.
  command_list *get_setup_commands () const   { return m_setup_commands; }
  command_list *get_profile_commands () const { return m_profile_commands; }
  command_list *get_compile_commands () const { return m_compile_commands; }
  command_list *get_run_commands () const     { return m_run_commands; }
  command_list *get_link_commands () const    { return m_link_commands; }
  command_list *get_check_commands () const   { return m_check_commands; }
  command_list *get_clean_commands () const   { return m_clean_commands; }

  // Call this function to expand variable definitions in
  // command lists.
  void expand_variable_definitions ();
    
  double optimizability () const { return m_optimizability; }
  double recency () const        { return m_recency; }
  time_t timeout () const        { return m_timeout; }
  int    bid () const            { return m_bid; }
  string name () const           { return m_name; }

  void set_benchname (string& n)     { m_name = n; }
  void set_timeout (time_t to)       { m_timeout = to; }
  void set_optimizability (double o) { m_optimizability = o; }
  void increment_recency ()          { m_recency += 1.0; }
  void reset_recency ()              { m_recency = 0.0; }

 protected:

  unsigned variable_length (char*) const;
  void     expand_commands (command_list*);
  void     expand_string   (string&);

 protected:

  static int k_bid;

  environment m_env;
  command_list *m_setup_commands;
  command_list *m_profile_commands;
  command_list *m_compile_commands;
  command_list *m_link_commands;
  command_list *m_run_commands;
  command_list *m_check_commands;
  command_list *m_clean_commands;

  string m_name;
  time_t m_timeout;
  double m_optimizability;
  double m_recency;
  int m_bid;
};


typedef vector<benchmark_info*> benchmark_vector;


#endif
