/****************************************************************************
 *
 *   File: benchmark_set.h:
 *   
 *   Author: Mark Stephenson
 *   Date: Wed Feb 12 15:21:12 2003
 *
 *   Function: The set of benchmarks that the harness will consider.
 *
 ***************************************************************************/

#ifndef _BENCHMARK_SET__H__
#define _BENCHMARK_SET__H__

#include <functional>
#include "benchmark_info.h"


struct bench_lt : 
  binary_function<const benchmark_info*, const benchmark_info*, bool>
{
  // Returns true if x is less than y.
  bool operator () (const benchmark_info* x, const benchmark_info* y)
  {
    double _x = x->recency() / x->optimizability();
    double _y = y->recency() / y->optimizability();
    return (_x > _y);
  }
};


class benchmark_set
{
 public:
  benchmark_set (char *benchmark_spec_fname);
  virtual ~benchmark_set () {};
  
  // Returns the number of benches in the set.
  virtual unsigned num_benches () const  
    { return m_benches.size(); }
  
  // Returns the bth benchmark.
  virtual benchmark_info* bench (unsigned b)
    { 
      assert (b < this->num_benches());
      return m_benches[b]; 
    }

  // Sorts the benchmarks according to fitness.
  virtual void sort ();

  // Prints the list of benchmarks.
  virtual void print (ostream&);
  virtual void summarize (ostream&);

  // Increments the recency of all benchmarks.
  virtual void increment_recencies ();

 protected:
  
  void print_commands (ostream&, char*, command_list*);

 protected:

  benchmark_vector m_benches;
};


#endif
