/*  Command-Line Parser Implementation */

/*  Copyright (c) 1994 Stanford University

    All rights reserved.

    This software is provided under the terms described in
    the "suif_copyright.h" include file. */

#include "cmdparse.h"
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <assert.h>


char *prog_ver_string;
char *prog_who_string;


/*
 *  Parse a command line and extract options we know about.
 */

static int
parse_cmd_line_int(char *s)
{
    int ret_int;
    
    if (s[0] == '0' && (s[1] == 'x' || s[1] == 'X'))
	sscanf(s+2, "%x", &ret_int);
    else
	ret_int = atoi(s);

    return ret_int;
}

void
parse_cmd_line (int &argc, char *argv[], cmd_line_option clos[], int nclos)
{
    int i, ap;

    int *elem_num; /* current position in array for CLO_MULTI_STRING */
    elem_num = new int[nclos];

    /*
     * First initialize the default values
     */

    for (i=0; i<nclos; i++) {
	switch (clos[i].knd) {
	case CLO_NOARG:
	    *(bool*)clos[i].data = false;
	    break;
	case CLO_INT:
	    *(int*)clos[i].data = parse_cmd_line_int(clos[i].deflt);
	    break;
	case CLO_DOUBLE:
	    *(double*)clos[i].data = atof(clos[i].deflt);
	    break;
	case CLO_STRING:
	    *(char**)clos[i].data = clos[i].deflt;
	    break;
	case CLO_MULTI_STRING:
	    elem_num[i] = 0;
	    (*(char ***)clos[i].data)[0] = clos[i].deflt;
	    break;
	default:
	  assert (false);
	}
    }

    /*
     * Now search through the argument list for matches.
     */

    bool foundone;
    
    for (ap=0; ap<argc;) {
	foundone = false;
	for (i=0; i<nclos; i++) {
	    if (!strcmp(argv[ap], clos[i].name)) {
		switch (clos[i].knd) {
		case CLO_NOARG:
		    *(bool*)clos[i].data = true;
		    break;
		case CLO_INT:
		    assert (ap != argc-1);
		    *(int*)clos[i].data = parse_cmd_line_int(argv[ap+1]);
		    break;
		case CLO_DOUBLE:
		    assert (ap != argc-1);
		    *(double*)clos[i].data = atof(argv[ap+1]);
		    break;
		case CLO_STRING:
		    assert (ap != argc-1);
		    *(char**)clos[i].data = const_string(argv[ap+1]);
		    break;
		case CLO_MULTI_STRING:
		    assert (ap != argc-1);
		    (*(char ***)(clos[i].data))[elem_num[i]] =
			    const_string(argv[ap+1]);
		    ++(elem_num[i]);
		    (*(char ***)(clos[i].data))[elem_num[i]] = clos[i].deflt;
		    break;
		}
		int ap2 = ap;
		if (clos[i].knd != CLO_NOARG)
		    ap2 = ap+2;
		else
		    ap2 = ap+1;
		for (int j=ap; ap2<=argc;) /* make sure to copy end NULL */
		    argv[j++] = argv[ap2++];
		if (clos[i].knd != CLO_NOARG)
		    argc -= 2;
		else
		    argc --;
		foundone = true;
		break;
	    }
	}
	if (!foundone)
	    ap++;
    }

    delete[] elem_num;
}


/*
 *  Make a constant string: In C++ there is no guarantee that a string
 *  passed to a routine will remain a constant since there may be other
 *  pointers to it.  When it is imperative that a string be a constant,
 *  const_string is called to create private copies of the strings.
 */

char *
const_string (char *s)
{
    return (s == NULL) ? NULL : strcpy(new char[strlen(s) + 1], s);
}
