%{

#include <stdio.h>
#include "benchmark_info.h"

extern FILE *_train_spec_in;
extern FILE *_train_spec_out;

benchmark_vector *_train_spec_bench_vec;
benchmark_info   *_train_spec_curr_bi;
int _train_spec_lineno;

extern char* _train_spec_text;

%}

%union {
  char* char_ptr_str;
  struct command_list* list;
};


%token START FINISH BENCH BENCHDIR SRCDIR 
%token SETUP PROFILE COMPILE LINK RUN CHECK
%token CLEAN SEMI EQUALS COMMA
%token <char_ptr_str> STRING VARIABLE UNKNOWN_TOKEN

%type <list> stringlist

%%
specs      :   specs spec
           |   spec
           ;
spec       :   START assigns FINISH {
		 _train_spec_bench_vec->push_back (_train_spec_curr_bi);
		 _train_spec_curr_bi = new benchmark_info;
               }
           ;
assigns    :   assigns assign
           |   assign
           ;
assign     :   VARIABLE EQUALS STRING SEMI { _train_spec_curr_bi->add_to_environment ($1, $3); }
           |   SETUP EQUALS stringlist SEMI { _train_spec_curr_bi->set_setup_commands ($3); }
           |   PROFILE EQUALS stringlist SEMI { _train_spec_curr_bi->set_profile_commands ($3); }
           |   COMPILE EQUALS stringlist SEMI { _train_spec_curr_bi->set_compile_commands ($3); }
           |   LINK EQUALS stringlist SEMI { _train_spec_curr_bi->set_link_commands ($3); }
           |   RUN EQUALS stringlist SEMI { _train_spec_curr_bi->set_run_commands ($3); }
           |   CHECK EQUALS stringlist SEMI { _train_spec_curr_bi->set_check_commands ($3); } 
           |   CLEAN EQUALS stringlist SEMI { _train_spec_curr_bi->set_clean_commands ($3); }
           ;
stringlist :   STRING COMMA stringlist { $$ = _train_spec_append_string ($1, $3); delete[] $1; }
           |   STRING { $$ = _train_spec_new_string ($1); delete[] $1; }
%%

extern "C"
{
  int yyparse (void);
  int yylex (void);  
  int yywrap ()
  {
    return 1;
  }
}

command_list*
_train_spec_append_string (char *s, command_list *list)
{
   command_list *tmp = new command_list;
   tmp->next = list;
   tmp->entry = s;

   return tmp;
}

command_list*
_train_spec_new_string (char *s)
{
   command_list *tmp = new command_list;
   tmp->next = NULL;
   tmp->entry = s;

   return tmp;
}

void
_train_spec_error (char *s)
{
  fprintf (stderr, "Line %d: %s at or before '%s'\n", 
	   _train_spec_lineno, s, _train_spec_text);
  exit (1);
}

void
invoke_train_spec_parser (FILE* input,
			  benchmark_vector* bench_vec)
{ 
  _train_spec_in = input;
  _train_spec_bench_vec = bench_vec;
  _train_spec_curr_bi = new benchmark_info;

  yyparse ();
  delete _train_spec_curr_bi;
}
