/****************************************************************************
 *
 *   File: proxycomp.cc:
 *   
 *   Author: Mark Stephenson
 *   Date: Tue Jun  3 15:49:40 2003
 *
 *   Function: This `fake' compiler actually tries to regression match.
 *
 ***************************************************************************/


#include "common.h"
#include "finch_interface.h"
#include <stdio.h>

// Prototypes for the 3 phases of our 1-pass compiler.
void initialize_compiler_pass ();
void run_compiler_algorithm ();
void finish_compiler_pass ();

// A helper function
double compute_function (bool, double, double);

int 
main (int argc, char** argv)
{
  // See the code in this function for the library call that
  // you should make when your compiler pass is initialized. This
  // should be in the same pass as the priority function that you
  // are trying to optimize.
  initialize_compiler_pass ();

  // See this code for the library call that you should make when
  // your algorithm actually runs (i.e., the priority function is
  // called).
  run_compiler_algorithm ();

  // See the library call that you should make to clean up any
  // dynamically allocated memory.
  finish_compiler_pass ();

  return 0;
}


void
initialize_compiler_pass ()
{
  FINCH_initialize_lib ();

  // Then do the rest of your pass' initialization.
  //      .
  //      .
  //      .
}


void 
run_compiler_algorithm ()
{
  double diff;
  double real_val;
  double sse = 0.0;

  // Pretend that this is the compiler heuristic that you're
  // hoping to optimize.  The heuristic likely performs legality 
  // checking and such, then makes several calls to a priority
  // function.

  // This "compiler" algorithm is just a test, so we're going
  // to do regression analysis instead.  The closest expression
  // to the function:
  // 
  // f = { i*j - j*j + 2.0  ;  if b = 0
  //     { i*i              ;  if b = 1
  // 
  // over the ranges [-10.0, 10.0] for i and j and {0,1} for b 
  // wins!

  for (int32 b = 0; b <= 1; b++)
    for (double i = -10.0; i <= 10.0; i += 1.0)
      for (double j = -10.0; j <= 10.0; j += 1.0) 
      {
	// Here's the way to evaluate finch's current priority function:
	double val = FINCH_evaluate_real_expr 
	  (4, 1,             // The # of dbl, bool args respectively.
	   i, j, 1.0, 0.5,   // The 4 double args.  
	   b);               // The 1 boolean arg.
	
	// Let's see how close we get with sum of squared errors.
	real_val = compute_function (b, i, j);
	diff = val - real_val;
	sse += diff * diff;
      }

  // When running a benchmark, a fitness of 0 is suspicious,
  // so 0's are actually flagged as errors.  Let's add 1 to the SSE:
  sse += 1;

  // Write the fitness out to disk.
  FILE *fitness = fopen ("fitness", "w");
  fprintf (fitness, "%f\n", sse);
  fclose (fitness);
}


void
finish_compiler_pass ()
{
  // After all calls to your priority function have been
  // performed, clean up dynamically allocated memory.
  FINCH_finalize_lib ();
}


double
compute_function (bool b, double i, double j)
{
  if (b == 0)
    return (i*j - j*j + 2.0);
  else
    return (i*i);
}

